/********************************************************************
--Execution Context and the EXECUTE AS Clause
*********************************************************************/
use bigpubs2005
go
sp_addlogin fred, fred2005
go
sp_grantdbaccess fred
go

create proc test_execute_as
with EXECUTE AS 'fred'
as
select user_name() as 'User context within proc'
go

select user_name() as 'User context before EXEC'
exec test_execute_as

/* -- results

User context before EXEC
-------------------------------
dbo

User context within proc
-------------------------------
Fred
*/


/********************************************************************
--Identifying Objects Referenced in Stored Procedures
*********************************************************************/

EXEC sp_depends titles
go

exec sp_depends title_authors
go


select * from information_schema.routines
where specific_catalog = 'bigpubs2005'
and specific_schema = 'dbo'
and routine_type = 'Procedure'
and routine_definition like '%tablename%'
go


select distinct object_name(object_id)
    from sys.sql_modules
    where definition like '%tablename%'
go


/********************************************************************
--Viewing Stored Procedures
*********************************************************************/

select routine_definition
from INFORMATION_SCHEMA.routines
where specific_catalog = 'bigpubs2005'
  and specific_schema = 'dbo'
  and routine_type = 'Procedure'
  and routine_name = 'title_authors'
go

/* -- results
routine_definition
---------------------------------------------------------------------------
CREATE PROCEDURE title_authors
AS
BEGIN
   SELECT a.au_lname, a.au_fname, t.title
      FROM titles t INNER JOIN
           titleauthor ta ON t.title_id = ta.title_id RIGHT OUTER JOIN
           authors a ON ta.au_id = a.au_id
   RETURN
END
*/


select definition
    from sys.sql_modules
    where object_id = object_id('title_authors')
go

/* -- results
CREATE PROCEDURE title_authors
AS
BEGIN
   SELECT a.au_lname, a.au_fname, t.title
      FROM titles t INNER JOIN
           titleauthor ta ON t.title_id = ta.title_id RIGHT OUTER JOIN
           authors a ON ta.au_id = a.au_id
   RETURN
END
*/

select object_definition(object_id('dbo.title_authors'))
go

/*
------------------------------------------------------------------------------
CREATE PROCEDURE title_authors @state char(2) = '%'
AS
BEGIN
   SELECT a.au_lname, a.au_fname, t.title
      FROM titles t INNER JOIN
           titleauthor ta ON t.title_id = ta.title_id RIGHT OUTER JOIN
           authors a ON ta.au_id = a.au_id
   RETURN
END
*/


/********************************************************************
--an outline of a stored procedure that presents the user with
-- information about the parameters expected if the user
-- doesn't pass parameters
*********************************************************************/


CREATE PROC MyProc2
 @parm1 int = NULL, @parm2 int = 32, @parm3 int = NULL
AS
IF (@parm1 IS NULL or @parm1 NOT BETWEEN 1 and 10) OR
   @parm3 IS NULL
PRINT 'Usage:
 EXEC MyProc2
 @parm1 int,   (Required: Can be between 1 and 10)
 @parm2 = 32,  (Optional: Default value of 32)
 @parm3 int,   (Required: Any number within range)'
-- Processing goes here
RETURN
GO

EXEC MyProc2
GO

Usage:
 EXEC MyProc2
 @parm1 int,   (Required: Can be between 1 and 10)
 @parm2 = 32,  (Optional: Default value of 32)
 @parm3 int,   (Required: Any number within range)



/********************************************************************
-- display stored procedure parameters by running a query against
-- the INFORMATION_SCHEMA view parameters
*********************************************************************/
select substring(Parameter_NAME,1, 30) as Parameter_name,
       substring (DATA_TYPE, 1, 20) as Data_Type,
       CHARACTER_MAXIMUM_LENGTH as Length,
       ordinal_position as param_order,
       Collation_name
from INFORMATION_SCHEMA.parameters
where specific_name = 'title_authors'
  and specific_schema = 'dbo'
order by ordinal_position

go

/*
Parameter_name   Data_Type     Length  param_order Collation_name
---------------- ------------- ------- ----------- ----------------------------
@state           char                2           1 SQL_Latin1_General_CP1_CI_AS
*/



/********************************************************************
-- Using Wildcards in Parameters
*********************************************************************/
IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'find_authors')
   DROP PROCEDURE dbo.find_authors
GO
 create proc find_authors @lastname varchar(40) = '%'
as
    select au_id, au_lname, au_fname
        from authors
        where au_lname like @lastname
        order by au_lname, au_fname
go

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'find_titles_by_sales')
   DROP PROCEDURE dbo.find_titles_by_sales
GO
create proc find_titles_by_sales @ytd_sales int = null
as
    select title_id, title, ytd_sales
        from titles
        where ytd_sales = isnull(@ytd_sales, ytd_sales)
go

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'find_titles_by_sales')
   DROP PROCEDURE dbo.find_titles_by_sales
GO
create proc find_titles_by_sales @ytd_sales int = null
as
if @ytd_sales is null
    select title_id, title, ytd_sales
        from titles
else
    select title_id, title, ytd_sales
        from titles
        where ytd_sales= @ytd_sales
go



/********************************************************************
-- Using Output Parameters
*********************************************************************/

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'ytd_sales')
   DROP PROCEDURE dbo.ytd_sales
GO
CREATE PROC ytd_sales
@title varchar(80), @ytd_sales int OUTPUT
AS
SELECT @ytd_sales = ytd_sales
   FROM titles
   WHERE title = @title
RETURN
go

DECLARE @sales_up_to_today  int
EXEC ytd_sales 'Life Without Fear', @sales_up_to_today OUTPUT
PRINT 'Sales this year until today''s date: ' +
      CONVERT(VARCHAR(10), @sales_up_to_today) + '.'
go

/*
Sales this year until today's date: 111.
*/

--You can also pass the output parameter by name:
DECLARE @sales_up_to_today  int
EXEC ytd_sales 'Life Without Fear',
      @ytd_sales = @sales_up_to_today OUTPUT
PRINT 'Sales this year until today''s date: ' +
      CONVERT(VARCHAR(10), @sales_up_to_today) + '.'
go



/********************************************************************
-- Using Cursors in Stored Procedures
*********************************************************************/
if object_id('p1') is not null
    drop proc p1
go
if object_id('p2') is not null
    drop proc p2
go
create proc p2
as
set nocount on 
-- fetch from global cursor defined in calling proc p1
fetch c1
return
go

create proc p1
as 
set nocount on
-- Declare global cursor
declare c1 cursor global for 
   select title_id, type from titles
open c1
fetch c1
exec p2
close c1
deallocate c1
go


exec p1
go

/*
title_id type
-------- ------------
BI0194   biography   

title_id type
-------- ------------
BI1408   biography   
*/

alter proc p1
as 
set nocount on
-- Declare local cursor
declare c1 cursor local for 
   select title_id, type from titles
open c1
fetch c1
exec p2
close c1
deallocate c1
go
exec p1
go

/*
title_id type
-------- ------------
BI0194   biography   

Msg 16916, Level 16, State 1, Procedure p2, Line 5
A cursor with the name 'c1' does not exist.
*/

alter proc p2
as
set nocount on 
-- Declare another local cursor with same name 'c1'
declare c1 cursor local for 
   select au_id, au_lname from authors
open c1
fetch c1
close c1
deallocate c1
return
go
exec p1
go

/*
title_id type
-------- ------------
BI0194   biography   

au_id       au_lname
----------- ----------------------------------------
047-43-0360 Michener
*/


alter proc p2
as
set nocount on 
-- declare global cursor c2
declare c2 cursor global for 
   select au_id, au_lname from authors
open c2
--do not close/deallocate cursor so it can be used by calling proc p1
return
go
alter proc p1
as 
set nocount on
declare c1 cursor local for 
   select title_id, type from titles
open c1
fetch c1
exec p2
-- fetch from global cursor declared in proc p2
fetch c2
close c1
deallocate c1
close c2
deallocate c2
return
go
exec p1
go


/*
title_id type
-------- ------------
BI0194   biography   

au_id       au_lname
----------- ----------------------------------------
047-43-0360 Michener
*/


alter proc p1
as 
set nocount on
declare c1 cursor local for 
   select title_id, type from titles
open c1
fetch c1
exec p2
-- fetch from global cursor declared in proc p2
fetch c2
close c1
deallocate c1
-- Cursor c2 is not closed/deallocated before return
return
go
exec p1
go
--Cursor c2 should still be open here so the following fetch will work
fetch c2
go

/*
title_id type
-------- ------------
BI0194   biography   

au_id       au_lname
----------- ----------------------------------------
047-43-0360 Michener

au_id       au_lname
----------- ----------------------------------------
052-04-3539 Gray
*/



/********************************************************************
-- Using Cursors Variables in Stored Procedures
*********************************************************************/
IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'cursor_proc')
   DROP PROCEDURE dbo.cursor_proc
GO
create proc cursor_proc @cursor CURSOR VARYING OUTPUT
as
declare curs1 cursor global for
   select cast(title as varchar(30)) as title , pubdate from titles
set @cursor = curs1
open curs1
return
go


set nocount on
declare @curs CURSOR
exec cursor_proc @cursor = @curs output
go

/*
Msg 16915, Level 16, State 1, Procedure cursor_proc, Line 4
A cursor with the name 'curs1' already exists.
Msg 16905, Level 16, State 1, Procedure cursor_proc, Line 6
The cursor is already open.
*/

close curs1
deallocate curs1
go

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'cursor_proc')
   DROP PROCEDURE dbo.cursor_proc
GO
go
create proc cursor_proc @cursor CURSOR VARYING OUTPUT
as
if CURSOR_STATUS('global', 'curs1') = -3  -- cursor does not exist
    declare curs1 cursor global for
       select cast(title as varchar(30)) as title , pubdate from titles
if CURSOR_STATUS('global', 'curs1') = -1  -- cursor is not open
    open curs1
set @cursor = curs1
return
go


set nocount on
declare @curs CURSOR
exec cursor_proc @cursor = @curs output
fetch curs1
fetch @curs
-- close the cursor
close curs1
-- try to fetch from the cursor variable
fetch @curs
go

/*
title                          pubdate
------------------------------ -----------------------
Samuel Johnson                 1995-09-19 00:00:00.000

title                          pubdate
------------------------------ -----------------------
Freud, Dora, and Vienna 1900   1995-02-25 00:00:00.000

Msg 16917, Level 16, State 2, Line 7
Cursor is not open.
*/


/********************************************************************
-- The following example demonstrates the creation 
--   of an index on a temporary table:
*********************************************************************/
use bigpubs2005
go
create proc ptemp1 WITH RECOMPILE
as
select title_id, type, pub_id, ytd_sales
   into #temp_titles
   from titles

create index tmp on #temp_titles(pub_id)

select sum(ytd_sales)
   from #temp_titles
   where pub_id = '0736'
select min(ytd_sales)
   from #temp_titles
   where pub_id = '0736'

return
go


/********************************************************************
-- The following is a simple example that shows how to use 
--   a table variable in a stored procedure:
*********************************************************************/
-- proc to get year-to-date sales for all books published since specified date
-- with ytd_sales greater than specified threshold
create proc tab_var_test @pubdate datetime = null,
                         @sales_minimum int = 0
as

declare @ytd_sales_tab TABLE (title_id char(6),
                              title varchar(50),
                              ytd_sales int)

if @pubdate is null
    -- if no date is specified, set date to last year
    set @pubdate = dateadd(month, -12, getdate())

insert @ytd_sales_tab
    select title_id, convert(varchar(50), title), ytd_sales
      from titles
      where pubdate > @pubdate
        and ytd_sales > @sales_minimum
select * from @ytd_sales_tab
return
go

exec tab_var_test '6/1/1991', 10000
go

/*
title_id title                                              ytd_sales
-------- -------------------------------------------------- -----------
BU2075   You Can Combat Computer Stress!                    18722
MC3021   The Gourmet Microwave                              22246
TC4203   Fifty Years in Buckingham Palace Kitchens          15096
*/



-- function to get year to date sales for all books published since specified
-- date with ytd_sales greater than specified threshold
create function tab_function (@pubdate datetime = NULL,
                             @sales_minimum int = 0)
returns @ytd_sales_tab TABLE (title_id char(6),
                              title varchar(50),
                              ytd_sales int)
as
begin

if @pubdate is null
    -- if no date is specified, set date to last year
    set @pubdate = dateadd(month, -12, getdate())

insert @ytd_sales_tab
    select title_id, convert(varchar(50), title), ytd_sales
      from titles
      where pubdate > @pubdate
        and ytd_sales > @sales_minimum
return
end
go

declare @local_tab table (title_id char(6), title varchar(50), ytd_sales int)
insert @local_tab select * from tab_function('6/1/1991', 10000)
select * from @local_tab
go
/*
title_id title                                              ytd_sales
-------- -------------------------------------------------- -----------
BU2075   You Can Combat Computer Stress!                    18722
MC3021   The Gourmet Microwave                              22246
TC4203   Fifty Years in Buckingham Palace Kitchens          15096
*/


/********************************************************************
-- stored procedure to create and populate a temporary table:
*********************************************************************/
create proc recomp_test
as
create table #titles (title_id varchar(6), title varchar(80), pubdate datetime)
insert #titles select title_id, title, pubdate from titles
where pubdate >  '10/1/1991'
select * from #titles
go

exec recomp_test
go
create index idx1 on titles (pubdate)
go
exec recomp_test
go
drop index titles.idx1
go



/********************************************************************
-- Forcing Recompilation of Query Plans
*********************************************************************/

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'advance_range')
   DROP PROCEDURE dbo.advance_range
GO
create proc advance_range
    (@low money, @high money)
as
select * from dbo.titles
  where advance between @low and @high
return
go

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'advance_range')
   DROP PROCEDURE dbo.advance_range
GO
create proc advance_range
    (@low money, @high money)
    WITH RECOMPILE
as
select * from dbo.titles
  where advance between @low and @high
return
go

exec advance_range 5000, 10000 WITH RECOMPILE
go

IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'get_titles_data')
   DROP PROCEDURE dbo.get_titles_data
GO
create proc get_titles_data (@flag tinyint, @value money)
as
if @flag = 1
     select * from titles where price = @value
else
     select * from titles where advance = @value
go


IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'get_titles_data')
   DROP PROCEDURE dbo.get_titles_data
GO
drop proc get_titles_data
go
create proc get_titles_data_by_price (@value money)
as
     select * from titles where price = @value
go
create proc get_titles_data_by_advance (@value money)
as
     select * from titles where advance = @value
go
create proc get_titles_data (@flag tinyint, @value money)
as
if @flag = 1
     exec get_titles_data_by_price @value
else
     exec get_titles_data_by_advance @value
go


/********************************************************************
-- Using Dynamic SQL in Stored Procedures
*********************************************************************/
IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'get_order_data')
   DROP PROCEDURE dbo.get_order_data
GO
create proc get_order_data
 (@table varchar(30), @column varchar(30), @value int)
as
declare @query varchar(255)

select @query = 'select * from ' + @table
          + ' where ' + @column
          + ' = ' + convert(varchar(10), @value)

EXEC (@query)

return
go


/********************************************************************
-- Startup Procedures
*********************************************************************/

use master
go
create procedure good_morning
as
print 'Good morning, Dave'
return
go
sp_procoption good_morning, startup, true
go



use master
go
create procedure prime_cache
as
declare @tablename varchar(128)

declare c1 cursor for select name from pubs.dbo.sysobjects where type = 'U'
open c1
fetch c1 into @tablename
while @@fetch_status = 0
begin
    print 'Loading ''' + @tablename + ''' into data cache'
    exec ('select * from pubs.dbo.' + @tablename)
    fetch c1 into @tablename
end
close c1
deallocate c1
return
go

sp_procoption prime_cache, startup, true
go
