/********************************************************************
-- using a user-defined function, getonlydate(), to return the
--   current date, with a time of midnight
*********************************************************************/
CREATE TABLE Orders (
       OrderID int IDENTITY (1, 1) NOT NULL Primary Key,
       CustomerID nchar (5) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
       EmployeeID int NULL ,
       OrderDate datetime NULL default dbo.getonlydate(),
       RequiredDate datetime NULL ,
       ShippedDate datetime NULL
)
go

insert Orders (CustomerID, EmployeeID, RequiredDate)
    values ('BERGS', 3, dbo.getonlydate() + 7)
go

update Orders
     set ShippedDate = dbo.getonlydate()
     where OrderID = 1
go

select OrderDate,
       RequiredDate,
       ShippedDate
    from Orders
  where OrderDate = dbo.getonlydate()
go

/*
OrderDate               RequiredDate            ShippedDate
----------------------- ----------------------- -----------------------
2006-06-03 00:00:00.000 2006-06-10 00:00:00.000 2006-06-03 00:00:00.000
*/



/********************************************************************
-- examples of using a table-valued function that returns a list
--  of valid book types
*********************************************************************/
 
SELECT OrderDate,
       RequiredDate,
       ShippedDate
    from Orders
  where OrderDate >= convert(varchar(10), getdate(), 110)
    and OrderDate < convert(varchar(10), getdate() + 1, 110)
go

select * from dbo.valid_book_types()
go
insert titles
select * from newtitles
where type in (select * from dbo.valid_book_types())
go



/********************************************************************
-- variations of a function that returns the average price for a
--   specified type of book from the titles table:
*********************************************************************/
use bigpubs2005
go
CREATE FUNCTION AverageBookPrice(@booktype varchar(12) = '%')
RETURNS money
AS
BEGIN
    DECLARE @avg money
    SELECT @avg = avg(price)
    FROM titles
    WHERE type like @booktype
   
    RETURN @avg
END
go


CREATE FUNCTION AverageBookPrice2(@booktype varchar(12) = '%')
RETURNS money
AS
BEGIN
    RETURN ( SELECT avg(price)
             FROM titles
             WHERE type like @booktype)
END
go

select title_id, type, price from titles
where price > dbo.AverageBookPrice('popular_comp')
go

/*
title_id type         price
-------- ------------ ---------------------
PC1035   popular_comp 17.1675
PS2091   psychology   17.0884
*/


declare @avg1 money,
        @avg2 money,
        @avg3 money
select @avg1 = dbo.AverageBookPrice('popular_comp')
set @avg2 = dbo.AverageBookPrice('popular_comp')
exec @avg3 = dbo.AverageBookPrice 'popular_comp'
select @avg1 as avg1, @avg2 as avg2, @avg3 as avg3
go

/*
Warning: Null value is eliminated by an aggregate or other SET operation.
avg1                  avg2                  avg3
--------------------- --------------------- ---------------------
16.0643               16.0643               16.0643
*/



/********************************************************************
-- Inline Table-Valued Functions
*********************************************************************/
use bigpubs2005
go
CREATE FUNCTION AveragePricebyType (@price money = 0.0)
RETURNS table
AS
 
    RETURN ( SELECT type, avg(isnull(price, 0)) as avg_price
             FROM titles
             group by type
             having avg(isnull(price, 0)) > @price)
go

select * from AveragePricebyType (15.00)
go



/********************************************************************
-- Multi-statement Table-Valued Functions
*********************************************************************/
use bigpubs2005
go
CREATE FUNCTION AveragePricebyType2 (@price money = 0.0)
RETURNS @table table (type varchar(12) null, avg_price money null)
AS
begin
    insert @table
       SELECT type, avg(isnull(price,0)) as avg_price
             FROM titles
             group by type
             having avg(isnull(price, 0)) > @price
    return
end
go

select * from AveragePricebyType2 (15.00)
go

/*
type         avg_price
------------ ---------------------
business     15.0988
mod_cook     15.4236
*/



/********************************************************************
-- Creating T-SQl Functions
*********************************************************************/


ALTER FUNCTION AveragePricebyType2 (@price money = 0.0)
RETURNS @table table (type varchar(12) null, avg_price money null)
with schemabinding
AS
begin
    insert @table
       SELECT type, avg(price) as avg_price
             FROM dbo.titles
             group by type
             having avg(price) > @price
    return
end
go

alter table titles alter column price smallmoney null
go

/*
Msg 5074, Level 16, State 1, Line 1
The object 'AveragePricebyType2' is dependent on column 'price'.
Msg 5074, Level 16, State 1, Line 1
The statistics 'price' is dependent on column 'price'.
Msg 4922, Level 16, State 9, Line 1
ALTER TABLE ALTER COLUMN price failed because one or more objects access this column.
*/


CREATE FUNCTION striptime (@datetimeval datetime)
RETURNS datetime
AS
BEGIN
    DECLARE @dateval datetime
    SELECT @dateval = convert(char(10), isnull(@datetimeval, getdate()), 110)
    RETURN @dateval
END
GO



CREATE FUNCTION striptime2(@datetimeval datetime)
RETURNS datetime
WITH RETURNS NULL ON NULL INPUT
AS
BEGIN
    DECLARE @dateval datetime
    SELECT @dateval = convert(char(10), isnull(@datetimeval, getdate()), 110)
    RETURN @dateval

END
GO

/*
select dbo.striptime(NULL), dbo.striptime2(NULL)

----------------------- -----------------------
2006-06-05 00:00:00.000 NULL
*/

CREATE FUNCTION dbo.getonlydate()
RETURNS datetime
as
BEGIN
DECLARE @date datetime
SET @date = dbo.striptime( getdate())
RETURN @date
end
go



/********************************************************************
-- Using T-SQL to View Functions
*********************************************************************/

use bigpubs2005
go
exec sp_helptext getonlydate
go

/*
Text
------------------------------------------------------------------------------ 
create function getonlydate()
returns datetime
as
begin
declare @date datetime
set @date = convert(datetime, convert(char(10), getdate(), 110))
return @date
end
*/

SELECT routine_definition
from INFORMATION_SCHEMA.routines
where routine_name = 'getonlydate'
and specific_schema = 'dbo'
and specific_catalog = 'bigpubs2005'
go

/*
routine_definition
------------------------------------------------------------------------------
create function getonlydate()
returns datetime
as
begin
declare @date datetime
set @date = convert(datetime, convert(char(10), getdate(), 110))
return @date
end
*/

select substring(parameter_name,1,20) as parameter_name,
       substring(data_type, 1, 20) as data_type,
       Parameter_mode,
       ordinal_position
from INFORMATION_SCHEMA.parameters
where specific_name = 'striptime'
and specific_schema = 'dbo'
and specific_catalog = 'bigpubs2005'
order by ordinal_position
go

/*
parameter_name       data_type            Parameter_mode ordinal_position
-------------------- -------------------- -------------- ----------------
                     datetime             OUT            0
@datetimeval         datetime             IN             1

*/

select substring(column_name, 1, 20) as column_name,
       substring (data_type, 1, 12)
             + case when character_maximum_length is not null
                 then '(' + cast(character_maximum_length as varchar(4)) + ')'
                      else ''
                      end
              as datatype,
       numeric_precision as 'precision',
       numeric_scale as scale,
       ordinal_position
from INFORMATION_SCHEMA.routine_columns
where table_name = 'AveragePricebyType'
order by ordinal_position
go

/*
column_name          datatype           precision scale       ordinal_position
-------------------- ------------------ --------- ----------- ----------------
type                 char(12)           NULL      NULL        1
avg_price            money              19        4           2
*/


select convert(varchar(10), specific_Schema) as 'schema',
  convert(varchar(20), specific_name) as 'function', 
  case objectproperty(object_id(specific_name), 'IsScalarFunction')
      when 1 then 'Yes' else 'No' end as IsScalar,
  case objectproperty(object_id(specific_name), 'IsTableFunction')
      when 1 then 'Yes' else 'No' end as IsTable,
  case objectproperty(object_id(specific_name), 'IsInlineFunction')
      when 1 then 'Yes' else 'No' end as IsInline,
  case objectproperty(object_id(specific_name), 'IsSchemaBound')
      when 1 then 'Yes' else 'No' end as IsSchemaBnd,
  case objectproperty(object_id(specific_name), 'IsDeterministic')
      when 1 then 'Yes' else 'No' end as IsDtrmnstc
from information_Schema.routines
where routine_type = 'FUNCTION'
order by specific_name
go

/*
schema     function             IsScalar IsTable IsInline IsSchemaBnd IsDtrmnstc
---------- -------------------- -------- ------- -------- ----------- ----------
dbo        AverageBookPrice     Yes      No      No       No          No
dbo        AverageBookPrice2    Yes      No      No       No          No
dbo        AveragePricebyType   No       Yes     Yes      No          No
dbo        AveragePricebyType2  No       Yes     No       Yes         Yes
dbo        getdateonly          Yes      No      No       No          No
dbo        getonlydate          Yes      No      No       No          No
dbo        striptime            Yes      No      No       No          No
dbo        striptime2           Yes      No      No       No          No
*/



/********************************************************************
-- Using T-SQL to Modify Functions
*********************************************************************/
ALTER FUNCTION AveragePricebyType2 (@price money = 0.0)
RETURNS @table table (type varchar(12) null, avg_price money null)
with schemabinding
AS
begin
    insert @table
       SELECT type, avg(price) as avg_price
             FROM dbo.titles
             group by type
             having avg(price) > @price
        order by avg(price) desc
    return
end
go


/********************************************************************
-- Setting Function Execution Context
*********************************************************************/
ALTER FUNCTION [dbo].[AverageBookPrice2](@booktype varchar(12) = '%')
RETURNS money
WITH EXECUTE AS 'dbo'
AS
BEGIN
    RETURN ( SELECT avg(price)
             FROM titles
             WHERE type like @booktype)
END
GO




/********************************************************************
-- example of manually deploying a table-valued CLR function
*********************************************************************/

CREATE ASSEMBLY fn_EventLog
FROM 'F:\assemblies\fn_EventLog\fn_eventlog.dll'
WITH PERMISSION_SET = SAFE
GO
CREATE FUNCTION ShowEventLog(@logname nvarchar(100))
RETURNS TABLE (logTime datetime,
               Message nvarchar(4000),
               Category nvarchar(4000),
               InstanceId bigint)
AS
EXTERNAL NAME fn_EventLog.TabularEventLog.InitMethod
GO
SELECT * FROM dbo.ReadEventLog(N'System') as T
go


