/********************************************************************
-- use the @@trancount function to monitor
--   the current status of a transaction
********************************************************************/

BEGIN TRAN
   DELETE FROM publishers
   WHERE pub_id = '0736'
go

/*
Msg 547, Level 16, State 0, Line 2
The DELETE statement conflicted with the REFERENCE constraint
 "FK__pub_info__pub_id__2BDE8E15". The conflict occurred in database
 "bigpubs2005", table "dbo.pub_info", column 'pub_id'.
The statement has been terminated.
*/


select @@trancount
go

/*          
-----------
          1
*/


/********************************************************************
-- Example that begins a transaction in one batch and then asks
--  for user verification during a second batch. 
********************************************************************/

--First batch:
begin transaction
insert publishers (pub_id, pub_name, city, state)
    values ('1111', 'Joe and Marys Books', 'Northern Plains', 'IA')
if @@error = 0
    print 'publishers insert was successful. Please go on.'
else
    print 'publisher insert failed. Please roll back'
go

--Second batch:
update titles
    set pub_id = '1111'
    where pub_id = '1234'
delete authors
    where state = 'CA'
commit transaction
go

-- A better way to write the previous code:
begin transaction
insert publishers (pub_id, pub_name, city, state)
    values ('1111', 'Joe and Marys Books', 'Northern Plains', 'IA')
if @@error = 0
begin
    print 'publishers insert was successful. Continuing.'
    update titles
        set pub_id = '1111'
        where pub_id = '1234'
    delete authors
        where state = 'CA'
    commit transaction
end
else
begin
    print 'publisher insert failed. rolling back transaction'
    rollback transaction
end
go



/********************************************************************
-- Triggers and Transaction Nesting
********************************************************************/
use bigpubs2005
go
CREATE TRIGGER tD_employee ON employee
FOR DELETE
AS
   DECLARE @msg VARCHAR(255)

   SELECT @msg = 'Trancount in trigger = ' + CONVERT(VARCHAR(2), @@trancount)

   PRINT @msg

   RETURN
go

set nocount on
print 'Trancount before delete = ' + CONVERT(VARCHAR(2), @@trancount)
DELETE FROM employee WHERE emp_id = 'PMA42628M'
print 'Trancount after delete = ' + CONVERT( VARCHAR(2), @@trancount)
go

/*
Trancount before delete = 0
Trancount in trigger = 1
Trancount after delete = 0
*/


begin tran
print 'Trancount before delete = ' + CONVERT(VARCHAR(2), @@trancount)
DELETE FROM employee WHERE emp_id = 'PMA42628M'
print 'Trancount after delete = ' + CONVERT( VARCHAR(2), @@trancount)
commit tran
print 'Trancount after commit = ' + CONVERT( VARCHAR(2), @@trancount)
go
/*
This code gives the following results:
Trancount before delete = 1
Trancount in trigger = 1
Trancount after delete = 1
Trancount after commit = 0
*/

ALTER TRIGGER tD_employee ON employee
FOR DELETE
AS
print 'Trancount in trigger = ' + CONVERT(VARCHAR(2), @@trancount)

ROLLBACK TRAN

return
go


ALTER TRIGGER tD_employee ON employee
FOR DELETE
AS
print 'Trancount in trigger = ' + CONVERT(VARCHAR(2), @@trancount)

ROLLBACK TRAN

print 'Trancount in trigger after rollback = ' + CONVERT(VARCHAR(2), @@trancount)

return
go



/********************************************************************
-- Triggers and Multistatement Transactions
********************************************************************/
--The first statement is used to disable any previously created
--DDL triggers in the database which would prevent creating a new trigger.
DISABLE TRIGGER ALL ON titles
go
create trigger tr_titles_i on titles for insert as
declare @rows int  -- create variable to hold @@rowcount
select @rows = @@rowcount
if @rows = 0 return
if update(pub_id) and (select count(*)
        from inserted i, publishers p
        where p.pub_id = i.pub_id ) != @rows
  begin
        rollback transaction
        raiserror ('Invalid pub_id inserted', 16, 1)
  end
return
go

--disable the foreign key constraint on the titles table
alter table titles nocheck constraint FK__titles__pub_id__0F424F67
go

/* transaction inserts rows into a table */
begin tran add_titles
insert titles (title_id, pub_id, title)
       values ('XX1234', '0736', 'Tuning SQL Server')
insert titles (title_id, pub_id, title)
       values ('XX1235', 'abcd', 'Tuning SQL Server')
insert titles (title_id, pub_id, title)
       values ('XX1236', '0877', 'Tuning SQL Server')
commit tran
go

/*
Msg 50000, Level 16, State 1, Procedure tr_titles_i, Line 10
Invalid pub_id inserted
Msg 3609, Level 16, State 1, Line 4
The transaction ended in the trigger. The batch has been aborted.
*/



/********************************************************************
-- Using Savepoints in Triggers
********************************************************************/

alter trigger tr_titles_i on titles for insert as
declare @rows int  -- create variable to hold @@rowcount
select @rows = @@rowcount
if @rows = 0 return
save tran titlestrig
if update(pub_id) and (select count(*)
        from inserted i, publishers p
        where p.pub_id = i.pub_id ) != @rows
  begin
       rollback transaction titlestrig
       raiserror ('Invalid pub_id inserted', 16, 1)
  end
return
go

begin tran add_titles
insert titles (title_id, pub_id, title)
       values ('XX1234', '0736', 'Tuning SQL Server')
if @@error = 50000 -- roll back entire transaction and abort batch
   begin
   rollback tran add_titles
   return
   end
insert titles (title_id, pub_id, title)
       values ('XX1236', '0877', 'Tuning SQL Server')
 if @@error = 50000 -- roll back entire transaction and abort batch
   begin
   rollback tran add_titles
   return
   end
save tran keep_first_two  -- set savepoint for partial rollback
insert titles (title_id, pub_id, title)
       values ('XX1235', 'abcd', 'Tuning SQL Server')
 if @@error = 50000  -- roll back to save point, continue batch
   begin
   rollback tran keep_first_two
   end
commit tran
go


-- reenable the constraint on the titles table when you are finished testing:
alter table titles check constraint FK__titles__pub_id__0F424F67
go







