/********************************************************************************
** Evaluating Index Usefulness
********************************************************************************/

-- determine the selectivity based on the author's last name
select count(distinct au_lname) as '# unique',
   count(*) as '# rows',
   str(count(distinct au_lname) / cast (count(*) as real),4,2) as 'selectivity'
from authors
go

/*
# unique    # rows      selectivity
----------- ----------- -----------
160         172         0.93
*/

-- determine the selectivity of the state column:
select count(distinct state) as '# unique',
    count(*) '# rows',
    str(count(distinct state) / cast (count(*) as real),4,2) as 'selectivity'
from authors
go

/*
# unique    # rows      selectivity
----------- ----------- -----------
38          172         0.22
*/

--determine whether duplicate values in the state column are skewing the selectivity
select state,
       count(*) as numrows,
       count(*)/b.totalrows * 100 as percentage
from authors a,
     (select convert(numeric(6,2), count(*)) as totalrows from  authors) as b
group by state, b.totalrows
having count(*) > 1
order by 2 desc
go

/*
state numrows     percentage
----- ----------- ---------------------------------------
CA    37          21.5116200
NY    18          10.4651100
TX    15          8.7209300
OH    9           5.2325500
FL    8           4.6511600
IL    7           4.0697600
NJ    7           4.0697600
WA    6           3.4883700
PA    6           3.4883700
CO    5           2.9069700
LA    5           2.9069700
MI    5           2.9069700
MN    3           1.7441800
MO    3           1.7441800
OK    3           1.7441800
AZ    3           1.7441800
AK    2           1.1627900
IN    2           1.1627900
GA    2           1.1627900
MA    2           1.1627900
NC    2           1.1627900
NE    2           1.1627900
SD    2           1.1627900
VA    2           1.1627900
WI    2           1.1627900
WV    2           1.1627900
*/


/********************************************************************************
** Index Densities
********************************************************************************/

--Calculate the density for the au_lname column in the authors table
Select Density = 1.00/ (select count(distinct au_lname) from authors)
Go

/*
Density
---------------------------------------
0.0062500000000
*/

-- Calculate the density for the combination of the columns au_lname and au_fname is as follows:
Select Density = 1.00/ (select count(distinct au_lname + au_fname) from authors)
Go

/*
Density
----------------
0.0058139534883
*/


/********************************************************************************
** Estimating Rows Using Index Statistics
********************************************************************************/

--The row estimate is the index density for the specified title_id key (0.001858736) multiplied by the number of rows in the table:
select count(*) * 0.001858736 as 'Row Estimate'
from sales
go

/*
Row Estimate
-------------------
313.615231600
*/

--Looking at the index density for title_id, stor_id, you can see that it is nearly unique:
select count(*) * 5.99844E-06 as 'Row Estimate'
from sales

/*
Row Estimate
--------------
1.012086789
*/


/********************************************************************************
** Generating and Maintaining Index and Column Statistics
********************************************************************************/

-- to turn off the AutoStats behavior by using the sp_autostats system stored procedure
Exec sp_autostats 'authors', 'OFF', 'aunmind'
go

-- display the current setting for the table as well as the database
Exec sp_autostats 'authors'
Go
/*
Global statistics settings for [bigpubs2005]:
  Automatic update statistics: OFF
  Automatic create statistics: ON

Settings for table [authors]

Index Name                 AUTOSTATS Last Updated
-------------------------- --------- -------------------------
[UPKCL_auidind]            ON        2006-07-03 17:24:23.170
[aunmind]                  OFF       2006-07-03 17:24:23.263
[_WA_Sys_state_4AB81AF0]   ON        2006-07-03 17:24:23.263
[au_fname]                 ON        2006-07-03 17:24:23.263
[phone]                    ON        2006-07-03 17:24:23.310
[address]                  ON        2006-07-03 17:24:23.310
[city]                     ON        2006-07-03 17:24:23.340
[zip]                      ON        2006-07-03 17:24:23.387
[state_idx]                ON        2006-07-03 17:24:23.387
*/


/********************************************************************************
** Manually Updating Statistics
********************************************************************************/

-- To determine the last time statistics were updated on the authors table
select STATS_DATE(object_id('authors'), 1)
go

--get the index ID from sys.indexes for each index on a table by using the following query:
select name, index_id from sys.indexes
   Where object_id = object_id('table_name') and index_id > 0


/********************************************************************************
** Column-Level Statistics
********************************************************************************/

--retrieve a list of auto-generated column statistics 
SELECT cast(object_name(object_id) as varchar(30)) as 'table',
       cast (name as varchar(30)) as autostats
   FROM sys.stats
   WHERE auto_created = 1
     AND objectproperty (object_id, 'IsUserTable') = 1
go


/********************************************************************************
** Creating Statistics
********************************************************************************/

--To display a list of all column statistics, whether auto-generated or manually created
SELECT cast(object_name(object_id) as varchar(30)) as 'table',
       cast (name as varchar(30)) as name,
       stats_id
   FROM sys.stats
   WHERE objectproperty (object_id, 'IsUserTable') = 1
     and (auto_created = 1 or user_created = 1)
order by 1, 3
go


/********************************************************************************
** Index Maintenance
********************************************************************************/

--To view the current fill factor setting, you can run a query such as the following against the sys.indexes system catalog view:
select cast(name as varchar(30)) as name, index_id, fill_factor
   from sys.indexes
   where object_id = object_id('sales')
go

--Using ALTER TABLE to reorganize the clustered index on the sales table:
ALTER INDEX UPKCL_sales on sales REORGANIZE
go

-- display the fragmentation of the UPKCL_sales index on the sales table:
select str(s.index_id,3,0) as indid,
      left(i.name, 20) as index_name,
      left(index_type_desc, 20) as index_type_desc,
      index_depth as idx_depth,
      index_level as level,
      str(avg_fragmentation_in_percent, 5,2) as avg_frgmnt_pct,
      str(page_count, 10,0) as pg_cnt
  FROM sys.dm_db_index_physical_stats
      (db_id('bigpubs2005'), object_id('sales'),1, 0, 'DETAILED') s
    join sys.indexes i on s.object_id = i.object_id and s.index_id = i.index_id
go

--rebuild the index using the ALTER INDEX command:
ALTER INDEX UPKCL_sales on sales REBUILD
go

--display the fragmentation of the UPKCL_sales index on the sales table:
select str(s.index_id,3,0) as indid,
      left(i.name, 20) as index_name,
      left(index_type_desc, 20) as index_type_desc,
      index_depth as idx_depth,
      index_level as level,
      str(avg_fragmentation_in_percent, 5,2) as avg_frgmnt_pct,
      str(page_count, 10,0) as pg_cnt
  FROM sys.dm_db_index_physical_stats
      (db_id('bigpubs2005'), object_id('sales'),1, 0, 'DETAILED') s
    join sys.indexes i on s.object_id = i.object_id and s.index_id = i.index_id
go



/********************************************************************************
** Setting the Fill Factor
********************************************************************************/

--o apply a 50% fill factor to the leaf and non-leaf pages in a nonclustered index on title_id in the titles table
CREATE INDEX title_id_index on titles (title_id)
        with (FILLFACTOR = 50, PAD_INDEX = ON)
go


