/*************************************************************************
** The max Specifier
*************************************************************************/

--a local variable being defined using the varchar(max) data type:
declare @maxvar varchar(max)
go

--The remaining examples in this section make use of the following table to demonstrate
-- the differences between a varchar(max) column and a text column:
create table maxtest (maxcol varchar(max),
                      textcol text)
go
-- populate the columns with some sample data
insert maxtest
   select replicate('1234567890', 1000), replicate('1234567890', 1000)
go
-- the substring function works with both varchar(max) and text data types:
select substring (maxcol, 1, 10),
       substring (textcol, 1, 10)
   from maxtext
go

/*
maxcol     textcol
---------- ----------
1234567890 1234567890
*/

go
--a varchar(max) column can be used for string concatenation, the text data type cannot:
select substring('xxx' + maxcol, 1, 10) from maxtest
go

/*
----------
xxx1234567
*/

select substring('xxx' + textcol, 1, 10) from maxtest
go

/*
Msg 402, Level 16, State 1, Line 1
The data types varchar and text are incompatible in the add operator.
*/


/*************************************************************************
** TOP Enhancements
*************************************************************************/

--example of a local variable as the numeric expression for the TOP clause
-- to limit the number of rows returned by a SELECT statement:
declare @rows int
select @rows = 5
select top (@rows) * from sales
go

/*
stor_id ord_num              ord_date                qty    payterms  title_id
------- -------------------- ----------------------- ------ --------- --------
6380    6871                 1994-09-14 00:00:00.000 5      Net 60    BU1032
6380    722a                 1994-09-13 00:00:00.000 3      Net 60    PS2091
6380    ONFFFFFFFFFFFFFFFFFF 1994-08-09 00:00:00.000 852    Net 30    FI1980
7066    A2976                1993-05-24 00:00:00.000 50     Net 30    PC8888
7066    ONAAAAAAAAAA         1994-01-13 00:00:00.000 948    Net 60    CH2480
*/

-- Using a subquery as the numeric expression
SELECT TOP(SELECT COUNT(*)/DATEDIFF(month, MIN(pubdate), MAX(pubdate))
            FROM titles)
       title_id, pub_id, pubdate
FROM titles
ORDER BY pubdate DESC
go

/*
title_id pub_id pubdate
-------- ------ -----------------------
CH9009   9903   1996-05-31 00:00:00.000
PC9999   1389   1996-03-31 05:27:00.000
FI0375   9901   1995-09-24 00:00:00.000
DR4250   9904   1995-09-21 00:00:00.000
BI4785   9914   1995-09-20 00:00:00.000
BI0194   9911   1995-09-19 00:00:00.000
BI3224   9905   1995-09-18 00:00:00.000
FI0355   9917   1995-09-17 00:00:00.000
FI0792   9907   1995-09-13 00:00:00.000
*/

-- Not Using the WITH TIES option
SELECT TOP 4 price, title
   FROM titles
   ORDER BY price DESC
go

/*
price                 title
--------------------- --------------------------------------
17.1675               But Is It User Friendly?
17.0884               Is Anger the Enemy?
15.9329               Emotional Security: A New Algorithm
15.894                You Can Combat Computer Stress!
*/

-- Using the WITH TIES option
SELECT TOP 4 WITH TIES price, title
   FROM titles
   ORDER BY price DESC
go

/*
price                 title
--------------------- ---------------------------------------
17.1675               But Is It User Friendly?
17.0884               Is Anger the Enemy?
15.9329               Emotional Security: A New Algorithm
15.894                The Gourmet Microwave
15.894                You Can Combat Computer Stress!
*/

--Using SET ROWCOUNT statement to limit rows deleted
set rowcount 100
DELETE sales where ord_date < (select dateadd(year, 1, min(ord_date)) from sales)
set rowcount 0

-- Using TOP to limit rows Deleted or updated
DELETE top (100) sales
   where ord_date < (select dateadd(year, 1, min(ord_date)) from sales)

UPDATE top (100) titles
   set royalty = royalty * 1.25



/*************************************************************************
** Using the OUTPUT clause
*************************************************************************/
--using the OUTPUT clause to verify the rows being deleted, updated, or inserted:
begin tran
delete from sales_big output deleted.*
where sales_id in (select top 10 sales_id from sales_big order by ord_date)
rollback
go

/*
sales_id    stor_id ord_num  ord_date                qty    payterms     title_id
----------- ------- -------- ----------------------- ------ ------------ --------
168745      7067    P2121    1992-06-15 00:00:00.000 40     Net 30       TC3218
168746      7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC4203
168747      7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC7777
20          7067    P2121    1992-06-15 00:00:00.000 40     Net 30       TC3218
21          7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC4203
22          7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC7777
337470      7067    P2121    1992-06-15 00:00:00.000 40     Net 30       TC3218
337471      7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC4203
337472      7067    P2121    1992-06-15 00:00:00.000 20     Net 30       TC7777
506195      7067    P2121    1992-06-15 00:00:00.000 40     Net 30       TC3218
*/


/*************************************************************************
** The TABLESAMPLE Clause
*************************************************************************/

with sales_sample as
( select * from sales TABLESAMPLE (1 percent) )
select count(*) as numrows from sales_sample
go


--specifying an actual number of rows
with sales_sample as
( select * from sales TABLESAMPLE (1000 rows) )
select count(*) as numrows from sales_sample
go


--specify a greater number of rows than you actually need 
-- and use the TOP option to specify the number of rows you actually want
select top 1000 * from sales TABLESAMPLE (2000 rows)

--Specify the REPEATABLE option with a specified repeat_seed value. 
-- The REPEATABLE option causes a selected sample to be returned again
with sales_sample as
( select * from sales TABLESAMPLE (10 percent) repeatable (1) )
select count(*) as numrows from sales_sample
go

-- REPEATABLE specified with a different repeat_seed value
with sales_sample as
( select * from sales TABLESAMPLE (10 percent) repeatable (2) )
select count(*) as numrows from sales_sample
go

--Running the query again with repeat_seed of 1 returns the same result rows as previously
with sales_sample as
( select * from sales TABLESAMPLE (10 percent) repeatable (1) )
select count(*) as numrows from sales_sample
go
